"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fhirResourceBundle = void 0;

var _object = require("../utils/object");

var _reduxBundler = require("redux-bundler");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const RECEIVED = 'FHIR_RESOURCE_RECEIVED';
const LOADING = 'FHIR_RESOURCE_LOADING';
const DELETING = 'FHIR_RESOURCE_DELETING';
const DELETED = 'FHIR_RESOURCE_DELETED';
const UPDATING = 'FHIR_RESOURCE_UPDATING';
const CLEARED = 'FHIR_RESOURCES_CLEARED';
const emptyObj = {};

const createFhirResourceSelector = resourceName => {
  let lastResult;
  let lastResourceCollection;
  return state => {
    const resourceCollection = state.fhirResourceCache[resourceName] || emptyObj;

    if (resourceCollection === lastResourceCollection) {
      return lastResult;
    }

    const result = {};

    for (const key in resourceCollection) {
      /** @type {ResourceEntry} */
      const entry = resourceCollection[key];
      const {
        resource,
        activeOperation
      } = entry;

      if (activeOperation) {
        result[resource.id] = _objectSpread(_objectSpread({}, resource), {}, {
          activeOperation
        });
      } else {
        result[resource.id] = resource;
      }
    }

    lastResourceCollection = resourceCollection;
    lastResult = result;
    return result;
  };
};
/** @typedef {'delete' | 'update' | 'load' | 'deleted'} OperationStatus */

/**
 * @typedef ResourceEntry
 * @property {any} resource
 * @property {OperationStatus} activeOperation
 */

/**
 * @param {any} state
 * @param {{
 *   id: string
 *   resourceType: string
 * }} payload
 * @param {OperationStatus} status
 */


const updateState = (state, payload, status) => {
  const {
    id,
    resourceType
  } = payload;
  const resourceCollection = state[resourceType] || {};

  if (status === 'deleted') {
    return _objectSpread(_objectSpread({}, state), {}, {
      [resourceType]: (0, _object.omit)(resourceCollection, [id])
    });
  }

  const existingResource = resourceCollection[id] && resourceCollection[id].resource;
  return _objectSpread(_objectSpread({}, state), {}, {
    [resourceType]: _objectSpread(_objectSpread({}, resourceCollection), {}, {
      [id]: {
        resource: existingResource || {
          resourceType,
          id: id
        },
        activeOperation: status
      }
    })
  });
};

const fhirResourceBundle = {
  name: 'fhirResourceCache',
  getReducer: () => {
    const initial = {// Patient: {
      //   [id]: {
      //     resource: {{ patient resource }}
      //     activeOperations: {'delete' | 'update' | 'load'}
      //   }
      // },
      // Location: {...},
      // Appointment: {...},
      // Practitioner: {...}
    };
    return (state = initial, {
      type,
      payload
    }) => {
      if (type === CLEARED) {
        return initial;
      }

      if (type === RECEIVED) {
        const updatedState = _objectSpread({}, state);

        for (const id in payload) {
          const resource = payload[id];
          const {
            resourceType
          } = resource;
          updatedState[resourceType] = _objectSpread(_objectSpread({}, updatedState[resourceType]), {}, {
            [id]: {
              resource
            }
          });
        }

        return updatedState;
      }

      if (type === LOADING) {
        return updateState(state, payload, 'load');
      }

      if (type === DELETING) {
        return updateState(state, payload, 'delete');
      }

      if (type === DELETED) {
        return updateState(state, payload, 'deleted');
      }

      if (type === UPDATING) {
        return updateState(state, payload, 'update');
      }

      return state;
    };
  },
  init: store => {
    store.utilFindFhirResourceById = id => {
      const collections = store.selectFhirResourceCacheRaw();

      for (const key in collections) {
        const items = collections[key];
        const found = items[id];

        if (found) {
          return found.resource;
        }
      }

      return null;
    };
  },
  selectFhirResourceCacheRaw: state => state.fhirResourceCache,
  selectPatientsRaw: createFhirResourceSelector('Patient'),
  selectLocationsRaw: createFhirResourceSelector('Location'),
  selectPractitionersRaw: createFhirResourceSelector('Practitioner'),
  selectAppointmentsRaw: createFhirResourceSelector('Appointment'),
  selectCompositionsRaw: createFhirResourceSelector('Composition'),
  selectOrganizationsRaw: createFhirResourceSelector('Organization'),
  selectFhirHasResources: (0, _reduxBundler.createSelector)('selectFhirResourceCacheRaw', cache => {
    for (const key in cache) {
      const collection = cache[key];

      for (const id in collection) {
        // if there's anything... it's true
        return true;
      }
    }

    return false;
  }),
  selectFhirActiveOperations: state => state.fhirResourceCache.activeOperations,
  doClearFhirCache: () => ({
    type: CLEARED
  }),
  doDeleteFhirResource: id => ({
    fhirFetch,
    dispatch,
    store
  }) => {
    const resource = store.utilFindFhirResourceById(id);

    if (!resource) {
      throw Error('NOT FOUND');
    }

    dispatch({
      type: DELETING,
      payload: id
    });
    return fhirFetch("/".concat(resource.resourceType, "/").concat(id), {
      method: 'DELETE'
    }).then(() => {
      store.doRemoveFhirResource(id);
    });
  },
  doSaveFhirResource: resource => ({
    fhirFetch,
    store
  }) => {
    const {
      id,
      resourceType
    } = resource;
    let method = 'post';
    let url = "/".concat(resourceType);

    if (id) {
      url += '/' + id;
      method = 'put';
    }

    return fhirFetch(url, {
      method,
      body: JSON.stringify(resource)
    }).then(res => {
      store.doReceiveFhirResource(res);
      return res;
    });
  },
  doRemoveFhirResource: id => ({
    type: DELETED,
    payload: id
  }),
  doReceiveFhirResource: unknown => ({
    dispatch
  }) => {
    const {
      entry,
      resourceType
    } = unknown;

    if (entry && Array.isArray(entry)) {
      const resources = entry.map(item => item.resource);
      dispatch({
        type: RECEIVED,
        payload: (0, _object.arrayToObject)(resources)
      });
    } else if (resourceType) {
      dispatch({
        type: RECEIVED,
        payload: {
          [unknown.id]: unknown
        }
      });
    } else {
      throw Error('Unknown resource received');
    } // return what was passed in


    return unknown;
  },
  doFhirFetch: (...args) => ({
    fhirFetch
  }) => fhirFetch(...args),
  doFetchFhirResource: fhirUrl => ({
    dispatch,
    fhirFetch,
    store
  }) => {
    const [resourceType, maybeId] = fhirUrl.split('/');
    const [id, extraQueryParams] = maybeId.split('?');
    dispatch({
      type: LOADING,
      payload: {
        resourceType,
        id
      }
    }); // we have to do this as a search in order to include related resources

    return fhirFetch("/".concat(resourceType, "?_id=").concat(id, "&_include=*").concat(extraQueryParams ? '&' + extraQueryParams : '')).then(store.doReceiveFhirResource);
  }
};
exports.fhirResourceBundle = fhirResourceBundle;